/*
 * Simple example to show AVR timer interrupts...
 *
 * This program was tested on a Kanda "AVR Workshop" board with AT90S8515
 * CPU installed. This board has 8 LEDS which show the PORTB outputs. When
 * run, this program causes an incrementing binary count to be visible on
 * these leds.
 *
 * Timer0 is initialized to run at CK/1024, causing an overflow every
 * 262144 (1024*256) clocks. This causes an interrupt which increments the
 * global variable 'irq_count'. The mainline code constantly copies the
 * content of irq_count to PORTB, giving a continuous binary readout of
 * the number of interrupts which have occured.
 *
 * Make sure you read the notes in AVRINT.H carefully!
 *
 * Compile command: CCAVR INTAVR -fopz
 */

#include <avrio.h>			/* Standard I/O definitions */
#include <avrreg.h>			/* Register definitions */
#include <avrint.h>			/* Interrupt definitions */

unsigned char irq_count;	/* Counts the interrupts */

/*
 * Mainline code, set up timer & loop displaying the count
 */
main()
{
	irq_count = 0;			/* Start counter at zero */

	DDRB = 0xff;			/* Set PORTB to all output */

	TCCR0 = 5;				/* Timer0 = CK / 1024 */
	TIMSK |= 2;				/* Enable interrupt on Timer0 overflow */
	enable();				/* Enable global interrupts */

	/* Loop forever, displaying contents of 'irq_count' */
	for(;;)
		PORTB = irq_count;	/* Display lsb of irq_count */
}

/*
 * Real simple interrupt handler - just increment the counter
 */
INTERRUPT(_TIMER0OVF_) int_handler()
{
	++irq_count;			/* Increment count of interrupts */
}
