;
; Startup code for: AVR
;
; Copyright 1998-2005 Dave Dunfield
; All rights reserved.
;
; Note: The stack pointer of the AVR is not located directly in a CPU
; register, but is located in two I/O registers. This would require a
; lot of code to manipulate for the C addressable stack... To allow the
; generation of much more efficent code, we maintain separate 'Hardware'
; (CPU) and 'Software' (C addressable) stacks. The hardware stack will
; contain only function return addresses, and possibly a few temporary
; register pushs/pops used in the library or inline assembly code. The
; Software stack will contain all of the C local variables and arguments,
; as well as any temporary pushs/pops required by the C compiler.
;
?HSTACK	EQU	$025F		; Hardware stack (smaller)
?SSTACK	EQU	$0250		; Software stack (larger)
?RAM	EQU	$0060		; Uninitialized data goes here
	ORG	$0000		; Code goes here
	RJMP	?begin		; Launch startup code
; This statement reserves room for the 12 interrupt vectors which are
; showm in the Atmel AVR data book. If you are using an enhanced device
; with more interrupt vectors, you will have to increase this. For a
; 2313, you can reduce this to 10... If you are really tight on code
; space, and are not using all of the interrupts, you may be able to
; reduce this even further...
	DS	12*2		; Reserve room for interrupt vectors
; Initialize process - Setup stack pointers
?begin	LDI	R24,?HSTACK	; Get low hardware stack
	LDI	R25,=?HSTACK	; Get high hardware stack
	OUT	SP-$20,R24	; Set low hardware stack
	OUT	SP-$1F,R25	; Set high hardware stack
	LDI	R28,?SSTACK	; Set low software stack
	LDI	R29,=?SSTACK	; Set high software stack
; Some example setup code for I/O ports
; Setup port B
;	LDI	R24,%11111111	; Set as all outputs
;	OUT	DDRB-$20,R24	; Write to PORTB DDR
;	LDI	R24,%01010101	; Initial value
;	OUT	PORTB-$20,R24	; Write to PORTB data
; Setup port D
;	LDI	R24,%00000000	; Set as all inputs
;	OUT	DDRD-$20,R24	; Write to PORT D DDR
	CLR	R24		; Get zero
	STS	?heap,R24	; Initialize the heap
; Setup serial port
	LDI	R24,25		; Baud rate for 9600 & 4.0 Mhz
	OUT	UBRR-$20,R24	; Write it
	LDI	R24,%00011000	; No interrupts, enable RX+TX, 8bit
	OUT	UCR-$20,R24	; Write it
;
	CALL	main		; Execute main program
exit	RJMP	exit		; Re-execute if returns
$SE:1
; Segment 1 - Data stored in code space
$SE:2
; Segment 2 - Uninitialized global/static data
	ORG	?RAM
