;
; Runtime support functions for: AVR
;
; Sign extend R24 into R25
?sign	CLR	R25		; Assume 0
	TST	R24		; Check sign bit
	BRPL	?ret2		; Positive = OK
	SER	R25		; Set negative
nargs	RET
; Sign extend R22 into R23
?psign	CLR	R23		; Assume 0
	TST	R22		; Check sign bit
	BRPL	?ret2		; Positive = OK
	SER	R23		; Set negative
	RET
; Convert !0 to 0, and 0 to 1
?not	OR	R24,R25		; Is register zero?
	BREQ	?ret1		; Yes, set to 1
	RJMP	?ret0		; No, set to 0
; Shift left by R22 bits
?shl	TST	R22		; Zero?
	BREQ	?ret2		; Exit
	DEC	R22		; Reduce count
	LSL	R24		; Shift low
	ROL	R25		; Shift high
	RJMP	?shl		; And proceed
; Shift right by R22 bits
?shr	TST	R22		; Zero?
	BREQ	?ret2		; Zero
	DEC	R22		; Reduce count
	LSR	R25		; Shift high
	ROR	R24		; Shift low
	RJMP	?shr		; And proceed
; Return 1 if R24:5 <  R22:3 (unsigned)
?ult	CP	R24,R22		; Compare low
	CPC	R25,R23		; Compare high
	BRLO	?ret1		; It is lower
	RJMP	?ret0		; False
; Return 1 if R24:5 <= R22:3 (unsigned)
?ule	CP	R24,R22		; Compare low
	CPC	R25,R23		; Compare high
	BRLO	?ret1		; It is lower
	BREQ	?ret1		; It is equal
	RJMP	?ret0		; False
; Return 1 of R24:5 >  R22:3 (unsigned)
?ugt	CP	R24,R22		; Compare low
	CPC	R25,R23		; Compare high
	BRLO	?ret0		; It's lower
	BREQ	?ret0		; They are the same
	RJMP	?ret1		; Must be higher
; Return 1 of R24:5 >= R22:3 (unsigned)
?uge	CP	R24,R22		; Compare low
	CPC	R25,R23		; Compare high
	BRLO	?ret0		; Lower, false
	RJMP	?ret1		; True
; Return 1 if R24:5 == R22:3
?eq	CP	R24,R22		; Low Same?
	CPC	R25,R23		; High Same?
	BRNE	?ret0		; No, false
?ret1	CLR	R25		; Zero high
	LDI	R24,1		; Set low
	TST	R24		; Insure Z clear
	RET
; Return 1 if R24:5 != R22:3
?ne	CP	R24,R22		; Low same?
	CPC	R25,R23		; High same?
	BRNE	?ret1		; No, true
?ret0	CLR	R25		; Zero high
	CLR	R24		; Zero low
?ret2	RET
; Return 1 if R24:5 <  R22:3 (signed)
?lt	CP	R24,R22		; Compare low
	CPC	R25,R23		; Compare high
	BRLT	?ret1		; It is lower
	RJMP	?ret0		; False
; Return 1 if R24:5 <= R22:3 (signed)
?le	CP	R24,R22		; Compare low
	CPC	R25,R23		; Compare high
	BRLT	?ret1		; It is lower
	BREQ	?ret1		; It is equal
	RJMP	?ret0		; False
; Return 1 of R24:5 >  R22:3 (signed)
?gt	CP	R24,R22		; Compare low
	CPC	R25,R23		; Compare high
	BRLT	?ret0		; It's lower
	BREQ	?ret0		; They are the same
	RJMP	?ret1		; Must be higher
; Return 1 of R24:5 >= R22:3 (signed)
?ge	CP	R24,R22		; Compare low
	CPC	R25,R23		; Compare high
	BRLT	?ret0		; Lower, false
	RJMP	?ret1		; True
; Multiply R24:5 by R22:3
?mul	MOV	R26,R24		; Save low1
	MOV	R27,R25		; Save high1
	CLR	R24		; Zero result low
	CLR	R25		; Zero result high
?mul1	LSR	R27		; Shift high n1
	ROR	R26		; Shift low n1
	BRCC	?mul2		; No carry, no add
	ADD	R24,R22		; Add in low
	ADC	R25,R23		; Add in high
?mul2	MOV	R0,R26		; Get high
	OR	R0,R27		; Is it zero
	BREQ	?ret2		; We are done
	LSL	R22		; Shift low n2
	ROL	R23		; Shift high n2
	MOV	R0,R22		; Get low
	OR	R0,R23		; It it zero?
	BRNE	?mul1		; Continue
	RET
; Divide R24:5 by R22:3, result in R24:5, remainder in R26:7 (signed)
?sdiv	MOV	R26,R25		; Copy high divisor
	EOR	R26,R23		; Compare sign with dividend
	BST	R26,7		; Set T if result will be negative
	TST	R25		; Is divisor negative?
	BRPL	?sdiv1		; No, it is not
	RCALL	?sdiv3		; 2's compliment divisor
?sdiv1	TST	R23		; Is dividend negative?
	BRPL	?sdiv2		; No, it's not
	COM	R22		; 1's compliment low
	COM	R23		; 1's compliment high
	INC	R22		; Adjust to 2's compliment low
	BRNE	?sdiv2		; No overflow
	INC	R23		; Adjust to 2's compliment high
?sdiv2	RCALL	?div		; Perform the division
	BRTC	?ret2		; Answer remains positive
?sdiv3	COM	R24		; 1's compliment low
	COM	R25		; 1's compliment high
	ADIW	W,1		; Adjust to 2's compliment
	RET
; If you are short of memory, and do not need signed division, comment
; out the above '?sdiv' function, and uncomment the EQU below to create
; a dummy entry point making all division unsigned. Also, be sure to
; remove the '?smod' function as well.
;?sdiv	EQU	*		; Dummy entry if no signed division
; Divide R24:5 by R22:3, result in R24:5, remainder in R26:7 (Unsigned)
?div	CLR	R26		; Zero remainder low
	CLR	R27		; Zero remainder high
	LDI	R19,17		; Number of bits (+1)
?div1	BCLR	C		; Zero carry
?div2	ROL	R24		; Shift n1 low (carry in)
	ROL	R25		; Shift ni high (carry out)
	DEC	R19		; Reduce count
	BREQ	?ret2		; End of line
	ROL	R26		; Shift remainder low (carry in)
	ROL	R27		; Shift remainder high
	CP	R26,R22		; Compare low
	CPC	R27,R23		; Compare high
	BRLO	?div1		; Carry in a zero
	SUB	R26,R22		; Subtract low
	SBC	R27,R23		; Subtract high
	BSET	C		; Set carry
	RJMP	?div2		; And proceed
; Perform signed division, and get remainder into R24:5
?smod	MOV	R0,R25		; Save original divisor
	RCALL	?sdiv		; Perform signed division
	MOV	R24,R26		; Get low result
	MOV	R25,R27		; Get high result
	TST	R0		; Was divisor negative
	BRMI	?sdiv3		; Yes, remainder is negative
	RET
;?smod	EQU	*		; Dummy entry if no signed division
; Perform unsigned division, and get remainder in R24:5
?mod	RCALL	?div		; Perform divide
	MOV	R24,R26		; Copy low
	MOV	R25,R27		; Copy high
	RET
; Perform a switch statement (Z=Table, W=value)
?swit1	ADIW	Z,1		; Skip to next
?switch	LPM			; Get low address
	MOV	R26,R0		; Save low
	ADIW	Z,1		; Advance to next
	LPM			; Get high address
	MOV	R27,R0		; Save high
	ADIW	Z,1		; Advance to next
	OR	R0,R26		; End of table?
	BREQ	?swit3		; Yes, use default
	LPM			; Get low value
	ADIW	Z,1		; Skip to next
	CP	R0,R24		; Match acc?
	BRNE	?swit1		; No, skip to next
	LPM			; Get high value
	CP	R0,R25		; Match acc?
	BRNE	?swit1		; Yes, we have it
	MOV	R31,R27		; Set high address
?swit2	MOV	R30,R26		; Set low address
	IJMP			; Execute handler
?swit3	LPM			; Get low address
	MOV	R26,R0		; Save low
	ADIW	Z,1		; Slip to next
	LPM			; Get high address
	MOV	R31,R0		; Save high
	RJMP	?swit2		; And proceed
; Read byte(X) from CODE memory
?getcsx	ST	-Y,R31		; Save Z
	ST	-Y,R30		; ""
	MOV	R30,R26		; Get address
	MOV	R31,R27		; ""
	LPM			; Get data from memory
	ADIW	X,1		; Advance
	LD	R30,Y+		; Restore Z
	LD	R31,Y+		; ""
	RET
