; Shared format routine used by various printf() type functions.
;
; R0     = Used to access program memory
; R1     = Output size
; R2     = Format width (0=Free format)
; R3     = Temp
; R4     = Temp
; R16:17 = Output function pointer (passed by caller)
; R18    = Fill/Justify
; R19    = Temp
; R20:21 = Format pointer
; R22    = Numeric base
; R23    = Temp
; R24:25 = W: Misc. holding registers
; R26:27 = X: Argument list pointer (Passed by caller)
; R28:29 = Y: C stack pointer
; R30:31 = Z: Temp/Indexing
_format_ LD	R21,-X		; Get format string
	LD	R20,-X		; ""
; Process next character from the format string
?1	RCALL	?5		; Get Read byte from format
	TST	R24		; End of string
	BREQ	?6		; Handle it
	CPI	R24,'%'		; Special case?
	BREQ	?8		; Yes, handle it
	RCALL	?7		; Write to output
	RJMP	?1		; And proceed
; Pad with fill on left (R2=Format, R1=Width)
?2	TST	R18		; Left justify
	BRMI	?6		; No need
; Pad with fill (R2=Format, R1=Width)
?3	TST	R2		; Free format?
	BREQ	?6		; No need
	MOV	R23,R1		; Make a copy
?4	CP	R23,R2		; Already past width?
	BRSH	?6		; Yes, exit
	MOV	R24,R18		; Get character
	ANDI	R24,%01111111	; Save only character
	RCALL	?7		; Output character
	INC	R23		; Reduce length
	RJMP	?4		; Do them all
; Load one byte the the format string (R20:21)
?5	MOV	R30,R20		; Get format pointer
	MOV	R31,R21		; ""
	LPM			; Read from program
	ADIW	Z,1		; Advance to next
	MOV	R24,R0		; Prepare for output
	MOV	R20,R30		; Copy back
	MOV	R21,R31		; ""
?6	RET
; Call output function (R16:17)
?7	ST	-Y,R31		; Save register
	ST	-Y,R30		; Save register
	MOV	R30,R16		; Get output function
	MOV	R31,R17		; ""
	ICALL			; Execute
	LD	R30,Y+		; Restore
	LD	R31,Y+		; Restore
	RET
; Test for '-'. left justify
?8	RCALL	?5		; Get next character
	LDI	R18,' '		; Assume space
	CPI	R24,'-'		; Left justify
	BRNE	?9		; No, its OK
	ORI	R18,%10000000	; Set LJ bit
	RCALL	?5		; Get next char
?9	CPI	R24,'0'		; Zero fill?
	BRNE	?10		; No, its OK
	ORI	R18,'0'		; Convert to ZF
; Evaluate field width
?10	CLR	R2		; Zero width
?11	CPI	R24,'0'		; In range?
	BRLO	?12		; No, not decimal
	CPI	R24,'9'+1	; In range?
	BRSH	?12		; No, not decimal
	SUBI	R24,'0'		; Convert
	MOV	R25,R2		; Copy at x1
	LSL	R2		; x2
	LSL	R2		; x4
	ADD	R2,R25		; x5
	LSL	R2		; x10
	ADD	R2,R24		; Include new digit
	RCALL	?5		; Get next digit
	RJMP	?11		; And process it
; Interpret the format specifier
?12	LD	R31,-X		; Get low
	LD	R30,-X		; Get high
	CLR	R1		; Zero length
	LDI	R22,2		; Assume base 2
	CPI	R24,'b'		; Binary?
	BREQ	?13		; Handle it
	LDI	R22,8		; Assume base 8
	CPI	R24,'o'		; Octal?
	BREQ	?13		; Handle it
	LDI	R22,16		; Assume base 16
	CPI	R24,'x'		; Hexidecimal?
	BREQ	?13		; Handle it
	LDI	R22,10		; Assume base 10
	CPI	R24,'u'		; Unsigned?
	BREQ	?13		; Handle it
	CPI	R24,'d'		; Decimal?
	BRNE	?19		; No, try something else
	TST	R31		; Negative
	BRPL	?13		; No, do nothing
	COM	R30		; Complement
	COM	R31		; ""
	ADIW	Z,1		; 2's complement
	LDI	R24,'-'		; Get char
	RCALL	?7		; Output
	TST	R2		; Test width
	BREQ	?13		; Free format
	DEC	R2		; Adjust
; Output Number(X) using Base R2
?13	MOV	R24,R30		; Get low
	MOV	R25,R31		; Get high
?14	INC	R1		; Advance count
	CLR	R23		; Zero high
	MOV	R3,R26		; Save X low
	MOV	R4,R27		; Save X high
	CALL	?div		; Perform divide
	ST	-Y,R26		; Save value
	MOV	R26,R3		; Restore X low
	MOV	R27,R4		; Restore X high
	MOV	R23,R24		; Get low
	OR	R23,R25		; Is it zero?
	BRNE	?14		; Stack all digits
	RCALL	?2		; Pad on left?
	MOV	R23,R1		; Get length
; Unstack digits & output
?15	LD	R24,Y+		; Get data
	ADIW	W,'0'		; Convert
	CPI	R24,'9'+1	; In decimal range?
	BRLO	?16		; Yes, its OK
	ADIW	W,7		; Convert to HEX
?16	RCALL	?7		; Output
	DEC	R23		; Reduce count
	BRNE	?15		; Do them all
; Pad on right if enabled
?17	TST	R18		; Pad on right
	BRPL	?18		; No
	RCALL	?3		; Perform padding
?18	RJMP	?1		; Next character
; 's' string
?19	CPI	R24,'s'		; String
	BRNE	?23		; No, try next
?20	LD	R24,Z+		; Get char
	TST	R24		; End?
	BREQ	?21		; Yes, stop
	INC	R1		; Increase length
	RJMP	?20		; And proceed
?21	RCALL	?2		; Pad on left
	ADIW	X,2		; Reset pointer
	LD	R31,-X		; Get string pointer
	LD	R30,-X		; ""
?22	LD	R24,Z+		; Get char
	TST	R24		; End of string
	BREQ	?17		; Yes, exit
	RCALL	?7		; Output
	RJMP	?22		; And proceed
; 'r' string from ROM
?23	CPI	R24,'r'		; ROM string?
	BRNE	?27
?24	LPM			; Get byte from ROM
	ADIW	Z,1		; Advance to next
	TST	R0		; End?
	BREQ	?25		; Yes, stop
	INC	R1		; Increase length
	RJMP	?24		; And proceed
?25	RCALL	?2		; Left justify
	ADIW	X,2		; Backup to arg
	LD	R31,-X		; Get string
	LD	R30,-X		; ""
?26	LPM			; Get char
	ADIW	Z,1		; Advance
	TST	R0		; End of string?
	BREQ	?17		; Yes, exit
	MOV	R24,R0		; Copy for output
	RCALL	?7		; output
	RJMP	?26		; And proceed
; 'c' Character
?27	INC	R1		; Length is 1
	CPI	R24,'c'		; Character?
	BREQ	?28		; It's OK
	MOV	R30,R24		; Not known, use this char
?28	RCALL	?2		; Left justify
	MOV	R24,R30		; Get char back
	RCALL	?7		; Output
	RJMP	?17		; And exit
